/****************************************************************/
/**                                                            **/
/**                         HL2.C                              **/
/**                                                            **/
/**    Generates a distribution of density via fractal         **/
/**    geometry                                                **/
/**                                                            **/
/**                                   Annibal Hetem Junior     **/
/**                                         29/03/93           **/
/**                                                            **/
/****************************************************************/

#include <stdio.h>
#include <math.h>
 
/*****************************************************************/
/**          aliases                                            **/
/*****************************************************************/

#define real    double
#define rnd     ((real)random())
#define sqr(a)  ((a)*(a))
#define dist(a,b,c,x,y,z) (sqr(a-x)+sqr(b-y)+sqr(c-z))

/*****************************************************************/
/**          global variables and types                         **/
/*****************************************************************/

char *cloud_name="                                         ";
char *status_name="                                        ";
FILE *Cloud;                             /* cloud file     */
real A;                                  /* free parameter */
int N;                                   /* size of cloud  */
int num_elem=0;                          /* number of elements */
real maximum=-1e30,minimum=1e30;

/*****************************************************************/
/**          from "man 3 random"                                **/
/*****************************************************************/

init_random()
{
  /* Initialize an array and pass it in to initstate. */

  static long state1[32] = {
               3,
               0x9a319039, 0x32d9c024, 0x9b663182, 0x5da1f342,
               0x7449e56b, 0xbeb1dbb0, 0xab5c5918, 0x946554fd,
               0x8c2e680f, 0xeb3d799f, 0xb11ee0b7, 0x2d436b86,
               0xda672e2a, 0x1588ca88, 0xe369735d, 0x904f35f7,
               0xd7158fd6, 0x6fa6f051, 0x616e6b96, 0xac94efdc,
               0xde3b81e0, 0xdf0a6fb5, 0xf103bc02, 0x48f340fb,
               0x36413f93, 0xc622c298, 0xf5a42ab8, 0x8a88d77b,
               0xf5ad9d0e, 0x8999220b, 0x27fb47b9
               };

  unsigned seed;
  int n;

  seed = 1;
  n = 128;
  initstate(seed, (char *) state1, n);
  setstate(state1);
}

/*****************************************************************/
/**          recursive routine                                  **/
/*****************************************************************/

generate ( x0,x1,y0,y1,z0,z1,mass,ann )
int x0,x1,y0,y1,z0,z1;
real mass;
int ann;
{
  int xm,ym,zm,d[8];
  real m[8],sm[8];
  real a,b;
  real r[8],s[8],sum_r=0;
  int  i,j,k,max=0,min=0;

  if (ann == 1) printf ("\n");
  if (ann <= 3) printf ("%d",ann);

  if ((x1 - x0) > 1) {

    /* distribution of the mass */

    for (i=0; i<=7; i++) { r[i] = rnd;
                           if (r[i] > r[max]) max = i;
                           if (r[i] < r[min]) min = i; 
                           sum_r += r[i];               }
  
    a = (1.0 - A) / ( (8*A*r[max]) - (8*r[min]) + sum_r - (A*sum_r) ); 
    b = (1.0 - (a*sum_r)) / 8;

    for (i=0; i<=7; i++) { s[i] = a*r[i] + b; m[i] = s[i] * mass; }

    /* center of cube */

    xm = (x0 + x1) / 2;
    ym = (y0 + y1) / 2;
    zm = (z0 + z1) / 2;

    if (ann<=2) {

      /* sort of the masses */

      for (i=0,k=0; i<=7; i++) {
        for (j=0; j<=7; j++) if (m[k] < m[j]) k=j;
        sm[i] = m[k];
        m[k] = -1;
      }

      /* distances to center of cloud */

#define xdist(a,b,c) dist((a+xm)/2,(b+ym)/2,(c+zm)/2,N/2,N/2,N/2 );

      d[0] = xdist ( x0,y0,z0 );
      d[1] = xdist ( x1,y0,z0 );
      d[2] = xdist ( x0,y1,z0 );
      d[3] = xdist ( x1,y1,z0 );
      d[4] = xdist ( x0,y0,z1 );
      d[5] = xdist ( x1,y0,z1 );
      d[6] = xdist ( x0,y1,z1 );
      d[7] = xdist ( x1,y1,z1 );

      /* redistribution of mass according to distance */

      for (i=7,k=0; i>=0; i--) {
        for (j=0; j<=7; j++) if (d[k] <= d[j]) k=j;
        m[k] = sm[i];
        d[k] = -1;
      }
    }

    /* recursive calls */

    /* 0 */ generate ( x0,xm,y0,ym,z0,zm,m[0],ann+1 );
    /* 1 */ generate ( xm,x1,y0,ym,z0,zm,m[1],ann+1 );
    /* 2 */ generate ( x0,xm,ym,y1,z0,zm,m[2],ann+1 );
    /* 3 */ generate ( xm,x1,ym,y1,z0,zm,m[3],ann+1 );
    /* 4 */ generate ( x0,xm,y0,ym,zm,z1,m[4],ann+1 );
    /* 5 */ generate ( xm,x1,y0,ym,zm,z1,m[5],ann+1 );
    /* 6 */ generate ( x0,xm,ym,y1,zm,z1,m[6],ann+1 );
    /* 7 */ generate ( xm,x1,ym,y1,zm,z1,m[7],ann+1 );
               
  } 
  else /* append element to cloud file */
  {
    num_elem++;
    if (minimum > mass) minimum = mass;
    if (maximum < mass) maximum = mass;
    fprintf ( Cloud, "%d %d %d %.9E\n",z0,y0,x0,mass );
  }
}

/*****************************************************************/
/**          status                                             **/
/*****************************************************************/

status()
{
  FILE *status_file;

  status_file = fopen ( status_name, "w" );

  fprintf ( status_file, "file    %s\n",cloud_name );
  fprintf ( status_file, "size    %d\n",N );
  fprintf ( status_file, "A       %f\n",A );
  fprintf ( status_file, "n_elem  %d\n",num_elem );
  fprintf ( status_file, "max     %.9E\n",maximum );
  fprintf ( status_file, "min     %.9E\n",minimum );
  
  fclose ( status_file );

  printf ( "\n%s generated with %d elements.\n",
            cloud_name,num_elem );
}

/*****************************************************************/
/**          main procedure                                     **/
/*****************************************************************/
 
main (argc,argv)
int argc;
char *argv[];
{
  char *i,*j;

  if (argc != 4) {
               printf ( "Missing arguments: hl2 cloud_name side A\n" );
               return;
             }
  
  for (i=argv[1],j=cloud_name; (*i) && ((*i) != '.'); *j++ = *i++ );
  *j++ = '.'; *j++ = 'f'; *j++ = 'r'; *j++ = 'a'; *j = 0;

  for (i=argv[1],j=status_name; (*i) && ((*i) != '.'); *j++ = *i++ );
  *j++ = '.'; *j++ = 's'; *j++ = 't'; *j++ = 't'; *j = 0;

  N = atoi ( argv[2] );
  A = atof ( argv[3] );

  printf ("\n*** HL2 fractal model ***\n");
  printf ("cloud: %s\n",cloud_name );
  printf ("size: %d\n",N);
  printf ("A: %f\n",A);

  init_random();

  Cloud = fopen ( cloud_name, "w" );

  generate ( 0,N,0,N,0,N,1.0,0 );

  fclose ( Cloud );

  status ();
}
