/****************************************************************/
/**                                                            **/
/**                          PROJ.C                            **/
/**                                                            **/
/**    Calculates 2D projections from 3D distributions         **/
/**                                                            **/
/**                                   Annibal Hetem Junior     **/
/**                                         30/03/93           **/
/**                                                            **/
/****************************************************************/

#include <stdio.h>
#include <math.h>
 
/*****************************************************************/
/**          aliases                                            **/
/*****************************************************************/

#define real double
#define MAXS 124

/*****************************************************************/
/**          global variables and types                         **/
/*****************************************************************/

char *cloud_name="                                         ";
char *status_name="                                        ";
char *proj_name="                                          ";
FILE *Cloud;                             /* cloud file     */
int N;                                   /* side of cloud  */

/*****************************************************************/
/**          read status                                        **/
/*****************************************************************/

read_status ()
{
  char *a="                                         ";
  char *i,*j,b;
  FILE *status;
 
  status = fopen ( status_name,"r" );

  do {
    i = a;
    do {
      b = fgetc(status);
      if ((b) >= ' ') *i++ = b;
      *i = 0;
    } while (b >= ' ');
  
    if (strncmp ( a,"file",4 ) == 0) {
      for(i=cloud_name,j=a+8; (*j) > ' '; *i++ = *j++ );
      for(i=cloud_name; *i; i++) if (*i == ' ') *i=0; }

    if (strncmp ( a,"size",4 ) == 0)
      N = atoi ( a+8 );

  } while (*a);
  fclose ( status );
}

/*****************************************************************/
/**          read cloud and make one projection                 **/
/*****************************************************************/

projection ( name,axis )
char *name,axis;
{
  FILE *proj;
  real p[MAXS][MAXS];
  int x,y,z,i,j;
  real m; 

  printf("%s\n",name);
  Cloud = fopen ( cloud_name, "r" );
  
  for (i=0; i<N; i++) for (j=0; j<N; j++) p[i][j] = 0.0;
  
  do {
    fscanf ( Cloud, "%d%d%d%E",&z,&y,&x,&m );

    switch (axis) {
      case 'x' : p[y][z] += m; break;
      case 'y' : p[x][z] += m; break;
      case 'z' : p[x][y] += m; break;
    }

  } while (!feof(Cloud)); 
  fclose ( Cloud );

  proj = fopen ( name, "w" );
  for (i=0; i<N; i++) for (j=0; j<N; j++) 
    fprintf ( proj, "%d %d %g\n",i,j,p[i][j] );
  fclose ( proj );
}

/*****************************************************************/
/**          main procedure                                     **/
/*****************************************************************/
 
main (argc,argv)
int argc;
char *argv[];
{
  char *i,*k,*l;

  if (argc != 2) {
               printf ( "Missing arguments: proj cloud_name\n" );
               return;
             }
  
  for (i=argv[1],k=status_name,l=proj_name; 
      (*i) && ((*i) != '.'); *k++ = *i, *l++=*i, i++ );
  *k++ = '.'; *k++ = 's'; *k++ = 't'; *k++ = 't'; *k = 0;
  *l++ = '.'; *l = 'x'; *(l+1) = 0;

  read_status();

  printf ("\n*** 2D Projections from 3D distributions ***\n");
  printf ("cloud: %s  size=%d\n",cloud_name,N );

  if (N > MAXS) {
    printf ("size too large.\n");
    return;
  }

  projection ( proj_name, *l );
  *l = 'y';
  projection ( proj_name, *l );
  *l = 'z';
  projection ( proj_name, *l );

}
